/* CHK=0xC2C5 */
/*+-------------------------------------------------------------------------
	afterlint.c -- process -Zg output

Use with zgcc:

	#!/bin/csh
	#zgcc -- build prototyping "__STDC__"
	echo C sources in $1';' result file $2
	rm -f zg.t
	foreach i(`cat $1`)
	cc -DBUILDING_PROTOTYPES -Zg $argv[3-] $i > zg1.t
	sort zg1.t > zg2.t
	echo '/ *' $i '* /' >> zg.t    #remove spaces inside comment markers
	cat zg2.t >> zg.t
	end
	afterlint zg.t $2
	rm -f zg.t zg?.t
	echo Output in $2

--------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:07-25-1991-12:55-wht@n4hgf-ECU release 3.10 */
/*:01-09-1991-22:31-wht@n4hgf-ISC port */
/*:11-18-1990-21:15-wht@n4hgf-clobber 'extern  ' in prototypes */
/*:07-13-1988-19:50-wht-creation */

#include <stdio.h>

#define ff fprintf

#ifdef BSD
# include <sys/time.h>
#define strchr index
#define strrchr rindex
char *index();
char *rindex();
/**************************/
#else	/* assuming XENIX */
/**************************/
# include <time.h>
# include <sys/types.h>
# include <sys/timeb.h>
char *strchr();
char *strrchr();
#endif	/* system dependencies */

long time();
struct tm *localtime();

/*+-------------------------------------------------------------------------
	emit_editnote(fp)
--------------------------------------------------------------------------*/
void
emit_editnote(fp)
FILE	*fp;
{
	struct tm *ltime;
	long cur_time;
#ifdef XENIX
#ifdef MESSED_UP_DST
	struct timeb tp;
#endif
#endif

	cur_time = time((long *)0);

#ifdef XENIX
#ifdef MESSED_UP_DST
	ftime(&tp);				/* localtime() does not know about DST */
	if(tp.dstflag)			/* so we adjust with ftime, which does know */
		cur_time += 3600L;	/* problem in BSD too, but no ftime there */
#endif
#endif

	ltime = localtime(&cur_time);

	fputs("/*+:EDITS:*/\n",fp);

	ff(fp,"/*:%02d-%02d-%04d-%02d:%02d-afterlint-creation */\n",
	    ltime->tm_mon+1,ltime->tm_mday,ltime->tm_year + 1900,
	    ltime->tm_hour,ltime->tm_min);

}	/* end of emit_editnote */

/*+-------------------------------------------------------------------------
	main(argc,argv,envp)
--------------------------------------------------------------------------*/
main(argc,argv,envp)
int argc;
char **argv;
char **envp;
{
	register char *cptr;
	register FILE *fpin;
	register FILE *fpout;
	char buf[256];
	char *basename;

	if(argc < 2)
	{
		ff(stderr,"usage: afterlint <infile> [<outfile>]\n");
		ff(stderr,"if outfile not supplied, output is to stdout\n");
		exit(1);
	}

	if(!(fpin = fopen(argv[1],"r")))
	{
		perror(argv[1]);
		exit(1);
	}

	if(argc > 2)
	{
		if(!(fpout = fopen(argv[2],"w")))
		{
			perror(argv[2]);
			exit(1);
		}
		basename = argv[2];
	}
	else
		fpout = stdout;

	ff(fpout,
	    "/*+-----------------------------------------------------------------------\n");
	if(argc > 2)
		ff(fpout,"\t%s\n",basename);
	else
		ff(fpout,"\tfunction declarations\n",basename);
	ff(fpout,
	    "------------------------------------------------------------------------*/\n");

	emit_editnote(fpout);
	ff(fpout,"\n");
	ff(fpout,"#ifndef BUILDING_PROTOTYPES\n");
	ff(fpout,"#ifdef __STDC__\n\n");

	while(fgets(buf,sizeof(buf),fpin))
	{
		cptr = (strncmp(buf,"/*global*/  ",12)) ? buf : buf + 12;
		cptr = (strncmp(buf,"extern  ",8)) ? cptr : cptr + 8;
		cptr = (strncmp(buf,"static  ",8)) ? cptr : cptr + 8;
		fputs(cptr,fpout);
	}

	ff(fpout,"\n#else\t\t/* compiler doesn't know about prototyping */\n\n");

	fclose(fpin);
	fpin = fopen(argv[1],"r");

	while(fgets(buf,sizeof(buf),fpin))
	{
		cptr = (strncmp(buf,"/*global*/  ",12)) ? buf : buf + 12;
		cptr = (strncmp(buf,"extern  ",8)) ? cptr : cptr + 8;
		cptr = (strncmp(buf,"static  ",8)) ? cptr : cptr + 8;
		if(strncmp(cptr,"int ",4))
		{
			if(strchr(cptr,'('))
			{
				while(*cptr != '(')
					fputc(*cptr++,fpout);
				fputs("();\n",fpout);
			}
			else
				fputs(cptr,fpout);
		}
	}

	ff(fpout,"\n#endif /* __STDC__ */\n");
	ff(fpout,"#endif /* BUILDING_PROTOTYPES */\n");
	ff(fpout,"\n/* end of %s */\n",
	    (argc > 2) ? basename : "function declarations");

	fclose(fpin);
	fclose(fpout);
	exit(0);
}	/* end of main */

/* vi: set tabstop=4 shiftwidth=4: */
