/*+-------------------------------------------------------------------------
	ecuicmhelp.c -- help for icmd commands
	wht@n4hgf.Mt-Park.GA.US

  Defined functions:
	calculate_help_right_column()
	display_help_screen()
	display_help_stderr(cmd)
	help_cmd_line_setup(prompt)
	help_get_cmd()
	icmd_help(narg,arg)
	search_cmd_list_pcmd(cmd)
	show_cmds()

--------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:08-25-1991-14:39-wht@n4hgf-SVR4 port thanks to aega84!lh */
/*:08-01-1991-03:52-wht@n4hgf-when editing string, set cursor to end */
/*:07-25-1991-12:56-wht@n4hgf-ECU release 3.10 */
/*:08-14-1990-20:40-wht@n4hgf-ecu3.00-flush old edit history */

#include "ecucurses.h"
#define OMIT_TERMIO_REFERENCES
#define STDIO_H_INCLUDED
#include "ecu.h"
#define NEED_P_CMD
#include "ecucmd.h"
#include "ecukey.h"
#include "pc_scr.h"

#include "stdio_lint.h"

#define PDAT	"ecuhelp.data"

extern int rcvr_pid;

long start_pos[TOKEN_QUAN];
int start_pos_has_been_read = 0;
char help_filename[256] = "";
FILE *fpdat;		/* help data file */
int help_right_column = 0;	/* right column for show_cmds */
int help_longest_cmd = 0;
int help_longest_descr = 0;

/*+-------------------------------------------------------------------------
	search_cmd_list_pcmd(cmd)
--------------------------------------------------------------------------*/
P_CMD *
search_cmd_list_pcmd(cmd)
register char *cmd;
{
register P_CMD *cmd_list = icmd_cmds;

	while(cmd_list->token != -1)
	{
		if(minunique(cmd_list->cmd,cmd,cmd_list->mincnt))
			break;
		cmd_list++;
	}
	if(cmd_list->token == -1)
		return((P_CMD *)0);
	else
		return(cmd_list);

}	/* end of search_cmd_list_pcmd */

/*+-------------------------------------------------------------------------
	display_help_stderr(cmd)
--------------------------------------------------------------------------*/
void
display_help_stderr(cmd)
char *cmd;
{
register itmp;
P_CMD *pcmd;
char buf[128];

	if(! (pcmd = search_cmd_list_pcmd(cmd)))
	{
		ff(se,"'%s' is not a valid command\r\n",cmd);
		return;
	}

	if(!start_pos[pcmd->token])
	{
		ff(se,"no help available for '%s'\r\n",cmd);
		return;
	}

	fseek(fpdat,start_pos[pcmd->token],0);
	ff(se,"\r\n");
	while(fgets(buf,sizeof(buf),fpdat) != NULL)
	{
		itmp = strlen(buf);
		buf[--itmp] = 0;
		if(itmp == 0)
			break;
		ff(se,"%s\r\n",buf);
	}

}	/* end of display_help_stderr */

/*+-------------------------------------------------------------------------
	calculate_help_right_column()
--------------------------------------------------------------------------*/
void
calculate_help_right_column()
{
register itmp;
register P_CMD *pcmd = icmd_cmds;

	if(help_right_column)	/* already bee thru here? */
		return;				/* ... seems so */

	while(pcmd->token != -1)
	{
		if(!*pcmd->descr)
		{
			pcmd++;
			continue;
		}
		itmp = strlen(pcmd->cmd);
		if(itmp > help_longest_cmd)
			help_longest_cmd = itmp;
		itmp = strlen(pcmd->descr);
		if(itmp > help_longest_descr)
			help_longest_descr = itmp;
		pcmd++;
	}
	help_right_column = 1 + help_longest_cmd + 2 + help_longest_descr + 3;

}	/* end of calculate_help_right_column */

/*+-------------------------------------------------------------------------
	help_cmd_line_setup(prompt)
--------------------------------------------------------------------------*/
void
help_cmd_line_setup(prompt)
char *prompt;
{
register icol;
int y;
int x;

	wmove(stdscr,LINES - 1,0);
	wstandout(stdscr);
	waddch(stdscr,' ');
	waddch(stdscr,' ');
	waddstr(stdscr,prompt);
	getyx(stdscr,y,x);
	for(icol = x; icol < COLS-1; icol++)
		waddch(stdscr,' ');
	wmove(stdscr,y,x);
	wstandend(stdscr);
	wrefresh(stdscr);
}	/* end of help_cmd_line_setup */

/*+-------------------------------------------------------------------------
	char *help_get_cmd()
--------------------------------------------------------------------------*/
char *
help_get_cmd()
{
register y;
register x;
static char cmd[15];
char delim = 0;
int wgpos = -1;
int edit = 0;

	help_cmd_line_setup("Enter command name (ESC to quit):  ");
	getyx(stdscr,y,x);
	wstandout(stdscr);
	while((delim != ESC) && (delim != NL))
	{
		wingets(stdscr,y,x,cmd,sizeof(cmd) - 1,&delim,edit,&wgpos);
		edit = 1;
	}
	wstandend(stdscr);
	if((delim == ESC) || (cmd[0] == 0))
		return((char *)0);
	else
		return(cmd);
}	/* end of help_get_cmd */

/*+-------------------------------------------------------------------------
	display_help_screen()
get user command section choice and display that group of commands
--------------------------------------------------------------------------*/
int
display_help_screen()
{
register itmp;
register P_CMD *pcmd;
register y = 1;
register x = 0;
short cmdclass = 0;
char s80[80];
char **cpptr;
static char *list[] =
{
	"g   - general commands",
	"c   - communications-related commands",
	"t   - transfer-related commands",
	"p   - procedure-related commands",
    "Esc - exit help",
	(char *)0
};
static char keylist[] = {'g','c','t','p',ESC,0};


	wclear(stdscr);
	wmove(stdscr,0,0);
	wstandout(stdscr);
	waddstr(stdscr,"Interactive Command Help");
	getyx(stdscr,y,x);
	for(itmp = x; itmp < COLS-1; itmp++)
		waddch(stdscr,' ');
	wstandend(stdscr);

	itmp = 6;
	cpptr = list;
	while(*cpptr)
	{
		wmove(stdscr,itmp++,4);
		waddstr(stdscr,*cpptr++);
	}
	wmove(stdscr,19,4);
	waddstr(stdscr,"---- press a key -------");
	switch(winget_single(stdscr,"",keylist) & 0x7F)
	{
		case 'g': cmdclass = ccG; break;
		case 'c': cmdclass = ccC; break;
		case 't': cmdclass = ccT; break;
		case 'p': cmdclass = ccP; break;
		case ESC: return(1); /* <=================== */
	}

	pcmd = icmd_cmds;
	y = 1;
	x = 0;
	wmove(stdscr,y,x);
	wclrtobot(stdscr);
	while(pcmd->token != -1)
	{
		if(!*pcmd->descr || (pcmd->cmdclass != cmdclass))
		{
			pcmd++;
			continue;
		}
		wmove(stdscr,y,x);
		strcpy(s80,pcmd->cmd);
		pad_zstr_to_len(s80,help_longest_cmd + 2);
		for(itmp = 0; itmp < pcmd->mincnt; itmp++)
			s80[itmp] = to_upper(s80[itmp]);
		waddstr(stdscr,s80);

		strcpy(s80,pcmd->descr);
		pad_zstr_to_len(s80,help_longest_descr + 1);
		waddstr(stdscr,s80);

		if(!x)
#ifdef SVR4
			wvline(stdscr,sVR & 0x00ff, 1); 
#else
			waddch(stdscr,sVR);
#endif
		y++;
		if(y >= LINES - 2)
		{
			y = 1;
			x = help_right_column;
		}
		pcmd++;
	}
	wmove(stdscr,LINES - 2,0);
	wstandout(stdscr);
	waddstr(stdscr,
"Capitalized portion of listed command sufficient for command recognition");
	getyx(stdscr,y,x);
	for(itmp = x; itmp < COLS-1; itmp++)
		waddch(stdscr,' ');
	wstandend(stdscr);
	return(0);

}	/* end of display_help_screen */

/*+-------------------------------------------------------------------------
	show_cmds()
commands with null descriptions are "undocumented"
--------------------------------------------------------------------------*/
void
show_cmds()
{
register char *cptr;
int rcvr_active = (rcvr_pid > 0) || (rcvr_pid == -2);

	if(rcvr_active && (rcvr_pid != -2))
		kill_rcvr_process(SIGUSR1);

	calculate_help_right_column();
	windows_start();
	scrollok(stdscr,0);
	if(!display_help_screen())
	{
		while(cptr = help_get_cmd())
		{
			wmove(stdscr,LINES - 1,0);
			wclrtoeol(stdscr);
			wrefresh(stdscr);
			display_help_stderr(cptr);
			ff(se,"\r\npress return:  ");
			ttygetc(1);
			if(display_help_screen())
				break;
		}
	}
	windows_end(stdscr);
	redisplay_rcvr_screen();
	if(rcvr_active)
		start_rcvr_process(0);

}	/* end of show_cmds */

/*+-------------------------------------------------------------------------
	icmd_help(narg,arg)
--------------------------------------------------------------------------*/
void
icmd_help(narg,arg)
int narg;
char **arg;
{
register char *cptr;
char *getenv();

	ff(se,"\r\n");
	if(!help_filename[0])
	{
		if((cptr = getenv("ECUHELP")) == NULL)
			sprintf(help_filename,"%s/%s",ECULIBDIR,PDAT);
		else
			strcpy(help_filename,cptr);
	}

	if((fpdat = fopen(help_filename,"r")) == NULL)
	{
		perror(help_filename); fputc('\r',se);
		return;
	}

	if(!start_pos_has_been_read)
	{
		fread((char *)start_pos,sizeof(long),TOKEN_QUAN,fpdat);
		start_pos_has_been_read = 1;
	}

	if(narg > 1)
		display_help_stderr(arg[1]);
	else
		show_cmds();

	fclose(fpdat);
}	/* end of icmd_help */

/* vi: set tabstop=4 shiftwidth=4: */
